// Encoder.cpp: Implementierung der Klasse CEncoder.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "stdafx.h"
#include "Encoder.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#include "bit_output.c"
#include "fixed_model.c"
#include "adaptive_model.c"
#include "arithmetic_encode.c"
#include <assert.h>
#include <algorithm>




//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

CEncoder::CEncoder()
{
	setEncodeSource(NULL);
	setEncodeDestination(NULL);
}

CEncoder::~CEncoder()
{
	removeAllSteps();
}


void CEncoder::encode()
{
	assert(getEncodeSource()!=NULL);
	assert(getEncodeDestination()!=NULL);

	if(m_isAdaptiveModel){
		freq = adaptive_freq;
		start_model = adaptive_start_model;
		update_model = adaptive_update_model;

	}else{//static model
		freq = fixed_freq;
		start_model = fixed_start_model;
		update_model = fixed_update_model;
	}

	start_model();				/* Set up other modules.    */
    start_outputing_bits();
    start_encoding(this);

	int ch,symbol,count=0;
	while(m_encodeSource->hasMoreBytes()){
		ch = m_encodeSource->getNextByte();
		if(++count>32766)
			break;;
        symbol = char_to_index[ch];		/* Translate to an index.   */
        encode_symbol(symbol,cum_freq);		/* Encode that symbol.      */
        update_model(symbol);			/* Update the model.        */
    }

    encode_symbol(EOF_symbol,cum_freq);		/* Encode the EOF symbol.   */
    done_encoding();				/* Send the last few bits.  */
    done_outputing_bits();
}

void CEncoder::addEncodeObserver(CEncodeObserver *obs)
{
	m_encodeObservers.push_back(obs);
}

void CEncoder::removeAllEncodeObservers()
{
	m_encodeObservers.clear();
}



class CEncoder::ApplySources{
	private:
			int symbol;
	public:
		ApplySources(int s) : symbol(s){}
	
		void operator()(CEncodeObserver* obs){
			obs->observeSource(symbol);
		}

};
void CEncoder::applySourceObservers(int symbol)
{
	for_each(m_encodeObservers.begin(),m_encodeObservers.end(),ApplySources(symbol));	
}

class CEncoder::ApplyDestinations{
	private:
			int symbol;
	public:
		ApplyDestinations(int s) : symbol(s){}
	
		void operator()(CEncodeObserver* obs){
			obs->observeDestination(symbol);
		}

};
void CEncoder::applyDestinationObservers(int symbol)
{
	for_each(m_encodeObservers.begin(),m_encodeObservers.end(),ApplyDestinations(symbol));	

}

void CEncoder::addStep(CAritStep *step)
{
	m_steps.push_back(step);	
}


class CEncoder::DeleteStep{
public:
	void operator()(CAritStep* s){
		delete s;
	}
};

void CEncoder::removeAllSteps()
{
	for_each(m_steps.begin(),m_steps.end(),DeleteStep());
	m_steps.clear();
}
