/* ARITHMETIC ENCODING ALGORITHM. */

#include "arithmetic_coding.h"
#include "EncodeDestination.h"
#include "EncodeSource.h"
#include "AritStep.h"
#include <assert.h>

CEncodeDestination* dst;
CEncodeSource* src;
CEncoder* enc;
CAritStep* lastStep;

static void bit_plus_follow();	/* Routine that follows                     */


/* CURRENT STATE OF THE ENCODING. */

static code_value low, high;	/* Ends of the current code region          */
static long bits_to_follow;	/* Number of opposite bits to output after  */
				/* the next bit.                            */


/* START ENCODING A STREAM OF SYMBOLS. */

void start_encoding(CEncoder* d)
{   low = 0;					/* Full code range.         */
    high = Top_value;
    bits_to_follow = 0;				/* No bits to follow next.  */
	enc = d;
	dst = d->getEncodeDestination();
	src = d->getEncodeSource();
	lastStep = NULL;
}

/* OUTPUT BITS PLUS FOLLOWING OPPOSITE BITS. */

static void bit_plus_follow(int bit)
//    int bit;
{   output_bit(bit);				/* Output the bit.          */
    while (bits_to_follow>0) {
        output_bit(!bit);			/* Output bits_to_follow    */
        bits_to_follow -= 1;			/* opposite bits. Set       */
    }						/* bits_to_follow to zero.  */
}

/* ENCODE A SYMBOL. */


void encode_symbol(int symbol,int cum_freq[])
//    int symbol;			/* Symbol to encode                         */
//    int cum_freq[];		/* Cumulative symbol frequencies            */
{   long range;			/* Size of the current code region          */
	int verboseSymbol;
	assert(symbol>=0);
	assert(symbol<=No_of_symbols);
	if(symbol == No_of_symbols)
		verboseSymbol = No_of_symbols;
	 else verboseSymbol = index_to_char[symbol];
	CAritStep* step = new CAritStep();


	step->setSymbol(verboseSymbol);
	if(lastStep==NULL){
		step->setTop((double)high/(double)Top_value);
		step->setBottom(low);
	}

	enc->applySourceObservers(verboseSymbol);
    range = (long)(high-low)+1;
    high = low +				/* Narrow the code region   */
      (range*cum_freq[symbol-1])/cum_freq[0]-1;	/* to that allotted to this */
    low = low + 				/* symbol.                  */
      (range*cum_freq[symbol])/cum_freq[0];
	
	step->setUpper((double)high/(double)Top_value);
	step->setLower((double)low/(double)Top_value);
	step->setRelativeUpper((double)cum_freq[symbol-1]/(double)cum_freq[0]);
	step->setRelativeLower((double)cum_freq[symbol]/(double)cum_freq[0]);
//	step->setRelativeUpper((double)high/(double)Top_value);
//	step->setRelativeLower((double)low/(double)Top_value);

	if(lastStep!=NULL){
		lastStep->makeChildStep(step);
//		delete lastStep;
	}

	enc->addStep(step);

	lastStep = step;

    for (;;) {					/* Loop to output bits.     */
        if (high<Half) {
            bit_plus_follow(0);			/* Output 0 if in low half. */
			step->addBits(false,bits_to_follow+1);
        } 
        else if (low>=Half) {			/* Output 1 if in high half.*/
            bit_plus_follow(1);
			step->addBits(true,bits_to_follow+1);
            low -= Half;
            high -= Half;			/* Subtract offset to top.  */
        }
        else if (low>=First_qtr			/* Output an opposite bit   */
              && high<Third_qtr) {		/* later if in middle half. */
            bits_to_follow += 1;
            low -= First_qtr;			/* Subtract offset to middle*/
            high -= First_qtr;
        }
        else break;				/* Otherwise exit loop.     */

        low = 2*low;
        high = 2*high+1;			/* Scale up code range.     */
    }
}


/* FINISH ENCODING THE STREAM. */

void done_encoding()
{   bits_to_follow += 1;			/* Output two bits that     */
    if (low<First_qtr) bit_plus_follow(0);	/* select the quarter that  */
    else bit_plus_follow(1);			/* the current code range   */
}						/* contains.                */



